@@ -29,8 +29,11 @@ import android.app.ProgressDialog;
 import android.content.Context;
 import android.content.DialogInterface;
 import android.content.Intent;
+import android.content.pm.ActivityInfo;
 import android.content.res.Configuration;
 import android.graphics.Bitmap;
+import android.graphics.BitmapFactory;
+import android.net.Uri;
 import android.os.AsyncTask;
 import android.os.Bundle;
 import android.support.v13.app.FragmentStatePagerAdapter;
@@ -49,18 +52,19 @@ import butterknife.BindView;
 import butterknife.ButterKnife;
 import cx.ring.R;
 import cx.ring.application.RingApplication;
-import cx.ring.fragments.AccountCreationFragment;
 import cx.ring.fragments.AccountMigrationFragment;
-import cx.ring.fragments.HomeAccountCreationFragment;
-import cx.ring.fragments.ProfileCreationFragment;
-import cx.ring.fragments.RingAccountCreationFragment;
-import cx.ring.fragments.RingLinkAccountFragment;
+import cx.ring.account.HomeAccountCreationFragment;
+import cx.ring.account.ProfileCreationFragment;
+import cx.ring.account.RingAccountCreationFragment;
+import cx.ring.account.RingLinkAccountFragment;
+import cx.ring.fragments.SIPAccountCreationFragment;
 import cx.ring.model.Account;
 import cx.ring.model.AccountConfig;
 import cx.ring.model.ConfigKey;
-import cx.ring.model.DaemonEvent;
+import cx.ring.model.ServiceEvent;
 import cx.ring.services.AccountService;
 import cx.ring.services.DeviceRuntimeService;
+import cx.ring.utils.BitmapUtils;
 import cx.ring.utils.Log;
 import cx.ring.utils.Observable;
 import cx.ring.utils.Observer;
@@ -71,8 +75,9 @@ import ezvcard.parameter.ImageType;
 import ezvcard.property.FormattedName;
 import ezvcard.property.Photo;
 import ezvcard.property.RawProperty;
+import ezvcard.property.Uid;
 
-public class AccountWizard extends AppCompatActivity implements Observer<DaemonEvent> {
+public class AccountWizard extends AppCompatActivity implements Observer<ServiceEvent> {
     static final String TAG = AccountWizard.class.getName();
 
     private boolean mCreatingAccount = false;
@@ -82,8 +87,9 @@ public class AccountWizard extends AppCompatActivity implements Observer<DaemonE
 
     private boolean mLinkAccount = false;
 
-    private boolean mCreatedAccount = false;
     private boolean mCreationError = false;
+    private boolean mCreatedAccount = false;
+
     private Bitmap mPhotoProfile;
     private String mFullname;
     private String mUsername;
@@ -93,6 +99,8 @@ public class AccountWizard extends AppCompatActivity implements Observer<DaemonE
     private Account mAccount;
     private String mCreatedAccountId;
 
+    public static final String PROFILE_TAG = "Profile";
+
     @Inject
     AccountService mAccountService;
 
@@ -115,22 +123,58 @@ public class AccountWizard extends AppCompatActivity implements Observer<DaemonE
         mViewPager.setAdapter(new WizardPagerAdapter(getFragmentManager()));
         mViewPager.getAdapter().notifyDataSetChanged();
 
-        if (shouldPresentMigrationScreen()) {
-            mViewPager.setVisibility(View.GONE);
-            Bundle args = new Bundle();
-            Fragment fragment;
-            args.putString(AccountMigrationFragment.ACCOUNT_ID, getIntent().getData().getLastPathSegment());
-            fragment = new AccountMigrationFragment();
-            fragment.setArguments(args);
-            FragmentManager fragmentManager = getFragmentManager();
-            fragmentManager
-                    .beginTransaction()
-                    .replace(R.id.migration_container, fragment)
-                    .commit();
-        } else {
-            mViewPager.setOffscreenPageLimit(4);
-            mAccountType = getIntent().getAction() != null ? getIntent().getAction() : AccountConfig.ACCOUNT_TYPE_RING;
+        String accountToMigrate = null;
+        Intent intent = getIntent();
+        if (intent != null) {
+            mAccountType = intent.getAction();
+            Uri path = intent.getData();
+            if (path != null) {
+                accountToMigrate = path.getLastPathSegment();
+            }
         }
+        if (mAccountType == null) {
+            mAccountType = AccountConfig.ACCOUNT_TYPE_RING;
+        }
+
+        if (savedInstanceState == null) {
+            if (accountToMigrate != null) {
+                mViewPager.setVisibility(View.GONE);
+                Bundle args = new Bundle();
+                args.putString(AccountMigrationFragment.ACCOUNT_ID, getIntent().getData().getLastPathSegment());
+                Fragment fragment = new AccountMigrationFragment();
+                fragment.setArguments(args);
+                FragmentManager fragmentManager = getFragmentManager();
+                fragmentManager
+                        .beginTransaction()
+                        .replace(R.id.migration_container, fragment)
+                        .commit();
+            } else {
+                mViewPager.setOffscreenPageLimit(4);
+            }
+        } else {
+            mProfileFragment = (ProfileCreationFragment) getFragmentManager().getFragment(savedInstanceState, PROFILE_TAG);
+            mFullname = savedInstanceState.getString("mFullname");
+            byte[] bytes = savedInstanceState.getByteArray("mPhotoProfile");
+            if (bytes != null && bytes.length > 0) {
+                mPhotoProfile = BitmapFactory.decodeByteArray(bytes, 0, bytes.length);
+            }
+            mLinkAccount = savedInstanceState.getBoolean("mLinkAccount");
+        }
+    }
+
+    @Override
+    protected void onSaveInstanceState(Bundle outState) {
+        super.onSaveInstanceState(outState);
+        if (mProfileFragment.isAdded()) {
+            getFragmentManager().putFragment(outState, PROFILE_TAG, mProfileFragment);
+        }
+        outState.putString("mFullname", mFullname);
+        if (mPhotoProfile != null) {
+            ByteArrayOutputStream stream = new ByteArrayOutputStream();
+            mPhotoProfile.compress(Bitmap.CompressFormat.PNG, 100, stream);
+            outState.putByteArray("mPhotoProfile", stream.toByteArray());
+        }
+        outState.putBoolean("mLinkAccount", mLinkAccount);
     }
 
     @Override
@@ -145,8 +189,13 @@ public class AccountWizard extends AppCompatActivity implements Observer<DaemonE
         mAccountService.removeObserver(this);
     }
 
-    public boolean shouldPresentMigrationScreen() {
-        return getIntent().getData() != null && !TextUtils.isEmpty(getIntent().getData().getLastPathSegment());
+    @Override
+    protected void onDestroy() {
+        if (mProgress != null) {
+            mProgress.dismiss();
+            mProgress = null;
+        }
+        super.onDestroy();
     }
 
     @Override
@@ -210,7 +259,7 @@ public class AccountWizard extends AppCompatActivity implements Observer<DaemonE
     private HashMap<String, String> initRingAccountDetails() {
         HashMap<String, String> accountDetails = initAccountDetails();
         if (accountDetails != null) {
-            accountDetails.put(ConfigKey.ACCOUNT_ALIAS.key(), "Ring");
+            accountDetails.put(ConfigKey.ACCOUNT_ALIAS.key(), mAccountService.getNewAccountName(getText(R.string.ring_account_default_name).toString()));
             accountDetails.put(ConfigKey.ACCOUNT_HOSTNAME.key(), "bootstrap.ring.cx");
             accountDetails.put(ConfigKey.ACCOUNT_UPNP_ENABLE.key(), AccountConfig.TRUE_STR);
         }
@@ -243,26 +292,14 @@ public class AccountWizard extends AppCompatActivity implements Observer<DaemonE
         }
     }
 
-    public void initSipAccountCreation(String alias, String username, String password, String host) {
-        HashMap<String, String> accountDetails = initAccountDetails();
-
-        if (accountDetails != null) {
-            mFullname = alias;
-            accountDetails.put(ConfigKey.ACCOUNT_ALIAS.key(), alias);
-            if (!TextUtils.isEmpty(host)) {
-                accountDetails.put(ConfigKey.ACCOUNT_HOSTNAME.key(), host);
-                accountDetails.put(ConfigKey.ACCOUNT_USERNAME.key(), username);
-                accountDetails.put(ConfigKey.ACCOUNT_PASSWORD.key(), password);
-            }
-            createNewAccount(accountDetails);
-        }
-    }
-
     public void saveProfile(String accountID) {
         VCard vcard = new VCard();
         vcard.setFormattedName(new FormattedName(mFullname));
+        String ringId = mAccountService.getCurrentAccount().getUsername();
+        vcard.setUid(new Uid(ringId));
         ByteArrayOutputStream stream = new ByteArrayOutputStream();
         if (mPhotoProfile != null) {
+            BitmapUtils.reduceBitmap(mPhotoProfile, VCardUtils.VCARD_PHOTO_SIZE);
             mPhotoProfile.compress(Bitmap.CompressFormat.PNG, 100, stream);
             Photo photoVCard = new Photo(stream.toByteArray(), ImageType.PNG);
             vcard.removeProperties(Photo.class);
@@ -317,14 +354,14 @@ public class AccountWizard extends AppCompatActivity implements Observer<DaemonE
     }
 
     @Override
-    public void update(Observable observable, DaemonEvent event) {
-        if (event == null) {
+    public void update(Observable observable, ServiceEvent event) {
+        if (event == null || (AccountConfig.ACCOUNT_TYPE_SIP.equals(mAccountType))) {
             return;
         }
 
         switch (event.getEventType()) {
             case ACCOUNT_ADDED:
-                mCreatedAccountId = event.getEventInput(DaemonEvent.EventInput.ACCOUNT_ID, String.class);
+                mCreatedAccountId = event.getEventInput(ServiceEvent.EventInput.ACCOUNT_ID, String.class);
                 handleCreationState(event);
                 break;
             case REGISTRATION_STATE_CHANGED:
@@ -336,19 +373,19 @@ public class AccountWizard extends AppCompatActivity implements Observer<DaemonE
         }
     }
 
-    private void handleCreationState(final DaemonEvent event) {
+    private void handleCreationState(final ServiceEvent event) {
         RingApplication.uiHandler.post(new Runnable() {
             @Override
             public void run() {
 
-                String stateAccountId = event.getEventInput(DaemonEvent.EventInput.ACCOUNT_ID, String.class);
+                String stateAccountId = event.getEventInput(ServiceEvent.EventInput.ACCOUNT_ID, String.class);
 
                 if (!TextUtils.isEmpty(stateAccountId) && stateAccountId.equals(mCreatedAccountId)) {
-                    String newState = event.getEventInput(DaemonEvent.EventInput.STATE, String.class);
+                    String newState = event.getEventInput(ServiceEvent.EventInput.STATE, String.class);
 
                     mAccount = mAccountService.getAccount(mCreatedAccountId);
 
-                    if (mAccount.isRing() && (newState.isEmpty() || newState.contentEquals(AccountConfig.STATE_INITIALIZING))) {
+                    if (mAccount != null && mAccount.isRing() && (newState.isEmpty() || newState.contentEquals(AccountConfig.STATE_INITIALIZING))) {
                         return;
                     }
 
@@ -363,13 +400,15 @@ public class AccountWizard extends AppCompatActivity implements Observer<DaemonE
                         if (mAlertDialog != null && mAlertDialog.isShowing()) {
                             return;
                         }
-                        mAlertDialog = createAlertDialog(AccountWizard.this, newState);
+                        mAlertDialog = createRingAlertDialog(AccountWizard.this, newState);
                         if (mCreatedAccount) {
                             saveProfile(mAccount.getAccountID());
                             mAlertDialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
                                 @Override
                                 public void onDismiss(DialogInterface dialogInterface) {
                                     setResult(Activity.RESULT_OK, new Intent());
+                                    //unlock the screen orientation
+                                    setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_SENSOR);
                                     finish();
                                 }
                             });
@@ -385,47 +424,6 @@ public class AccountWizard extends AppCompatActivity implements Observer<DaemonE
         });
     }
 
-    private AlertDialog createAlertDialog(Context context, String state) {
-        if (mAccount.isRing()) {
-            return createRingAlertDialog(context, state);
-        }
-
-        return createSIPAlertDialog(context, state);
-    }
-
-    private AlertDialog createSIPAlertDialog(Context context, String state) {
-        AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(context);
-        dialogBuilder.setPositiveButton(android.R.string.ok, null);
-        switch (state) {
-            case AccountConfig.STATE_ERROR_GENERIC:
-            case AccountConfig.STATE_UNREGISTERED:
-                dialogBuilder.setTitle(R.string.account_sip_cannot_be_registered)
-                        .setMessage(R.string.account_sip_cannot_be_registered_message);
-                mCreationError = true;
-                break;
-            case AccountConfig.STATE_ERROR_NETWORK:
-                dialogBuilder.setTitle(R.string.account_no_network_title)
-                        .setMessage(R.string.account_no_network_message);
-                mCreationError = true;
-                break;
-            default:
-                dialogBuilder.setTitle(R.string.account_sip_success_title)
-                        .setMessage(R.string.account_sip_success_message);
-                mCreatedAccount = true;
-                mCreationError = false;
-                break;
-        }
-        if (!mCreatedAccount) {
-            dialogBuilder.setNegativeButton(R.string.account_sip_register_anyway, new DialogInterface.OnClickListener() {
-                public void onClick(DialogInterface dialog, int id) {
-                    setResult(Activity.RESULT_OK, new Intent());
-                    finish();
-                }
-            });
-        }
-        return dialogBuilder.show();
-    }
-
     private AlertDialog createRingAlertDialog(Context context, String state) {
         AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(context);
         dialogBuilder.setPositiveButton(android.R.string.ok, null);
@@ -510,36 +508,20 @@ public class AccountWizard extends AppCompatActivity implements Observer<DaemonE
         mCreatingAccount = true;
         mCreationError = false;
 
+        //orientation is locked during the create of account to avoid the destruction of the thread
+        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LOCKED);
         //noinspection unchecked
         new CreateAccountTask(this).execute(accountDetails);
     }
 
-    @Override
-    public void onActivityResult(int requestCode, int resultCode, Intent data) {
-        super.onActivityResult(requestCode, resultCode, data);
-
-        switch (requestCode) {
-            case ProfileCreationFragment.REQUEST_CODE_PHOTO:
-                if (resultCode == RESULT_OK && data != null) {
-                    mProfileFragment.updatePhoto((Bitmap) data.getExtras().get("data"));
-                }
-                break;
-            case ProfileCreationFragment.REQUEST_CODE_GALLERY:
-                if (resultCode == RESULT_OK && data != null) {
-                    mProfileFragment.updatePhoto(data.getData());
-                }
-                break;
-            default:
-                break;
-        }
-    }
-
     private class WizardPagerAdapter extends FragmentStatePagerAdapter {
 
         WizardPagerAdapter(FragmentManager fm) {
             super(fm);
             mHomeFragment = new HomeAccountCreationFragment();
-            mProfileFragment = new ProfileCreationFragment();
+            if (mProfileFragment == null) {
+                mProfileFragment = new ProfileCreationFragment();
+            }
         }
 
         @Override
@@ -553,7 +535,7 @@ public class AccountWizard extends AppCompatActivity implements Observer<DaemonE
             switch (position) {
                 case 0:
                     if (AccountConfig.ACCOUNT_TYPE_SIP.equals(mAccountType)) {
-                        return new AccountCreationFragment();
+                        return new SIPAccountCreationFragment();
                     } else {
                         return mHomeFragment;
                     }
