@@ -37,7 +37,7 @@
 #include <TargetConditionals.h>
 #endif
 
-#ifdef __ANDROID__
+#if defined(__ANDROID__) || (defined(TARGET_OS_IOS) && TARGET_OS_IOS)
 #include "client/ring_signal.h"
 #endif
 #ifdef _WIN32
@@ -175,7 +175,7 @@ create_pidfile()
 std::string
 expand_path(const std::string &path)
 {
-#if defined __ANDROID__ || defined RING_UWP || defined WIN32 || TARGET_OS_IPHONE
+#if defined __ANDROID__ || defined RING_UWP || defined WIN32 || (defined(TARGET_OS_IOS) && TARGET_OS_IOS)
     RING_ERR("Path expansion not implemented, returning original");
     return path;
 #else
@@ -290,11 +290,39 @@ writeTime(const std::string& path)
 #endif
 }
 
+bool isPathRelative(const std::string& path)
+{
+#ifndef _WIN32
+    return not path.empty() and not (path[0] == '/');
+#else
+    return not path.empty() and path.find(":") == std::string::npos;
+#endif
+}
+
+std::string
+getCleanPath(const std::string& base, const std::string& path)
+{
+    if (base.empty() or path.size() < base.size())
+        return path;
+    auto base_sep = base + DIR_SEPARATOR_STR;
+    if (path.compare(0, base_sep.size(), base_sep) == 0)
+        return path.substr(base_sep.size());
+    else
+        return path;
+}
+
+std::string
+getFullPath(const std::string& base, const std::string& path)
+{
+    bool isRelative {not base.empty() and isPathRelative(path)};
+    return isRelative ? base + DIR_SEPARATOR_STR + path : path;
+}
+
 std::vector<uint8_t>
-loadFile(const std::string& path)
+loadFile(const std::string& path, const std::string& default_dir)
 {
     std::vector<uint8_t> buffer;
-    std::ifstream file(path, std::ios::binary);
+    std::ifstream file(getFullPath(default_dir, path), std::ios::binary);
     if (!file)
         throw std::runtime_error("Can't read file: "+path);
     file.seekg(0, std::ios::end);
@@ -389,7 +417,7 @@ FileHandle::~FileHandle()
     }
 }
 
-#if defined(__ANDROID__) || defined(RING_UWP)
+#if defined(__ANDROID__) || defined(RING_UWP) || (defined(TARGET_OS_IOS) && TARGET_OS_IOS)
 static std::string files_path;
 static std::string cache_path;
 static std::string config_path;
@@ -423,7 +451,7 @@ get_cache_dir()
         return cache_home;
     } else {
 #endif
-#ifdef __ANDROID__
+#if defined(__ANDROID__) || (defined(TARGET_OS_IOS) && TARGET_OS_IOS)
         std::vector<std::string> paths;
         emitSignal<DRing::ConfigurationSignal::GetAppDataPath>("cache", &paths);
         if (not paths.empty())
@@ -445,7 +473,7 @@ get_cache_dir()
 std::string
 get_home_dir()
 {
-#if defined __ANDROID__
+#if defined(__ANDROID__) || (defined(TARGET_OS_IOS) && TARGET_OS_IOS)
     std::vector<std::string> paths;
     emitSignal<DRing::ConfigurationSignal::GetAppDataPath>("files", &paths);
     if (not paths.empty())
@@ -489,7 +517,7 @@ get_home_dir()
 std::string
 get_data_dir()
 {
-#ifdef __ANDROID__
+#if defined(__ANDROID__) || (defined(TARGET_OS_IOS) && TARGET_OS_IOS)
     std::vector<std::string> paths;
     emitSignal<DRing::ConfigurationSignal::GetAppDataPath>("files", &paths);
     if (not paths.empty())
@@ -525,17 +553,13 @@ get_data_dir()
 std::string
 get_config_dir()
 {
-#ifdef __ANDROID__
+#if defined(__ANDROID__) || (defined(TARGET_OS_IOS) && TARGET_OS_IOS)
     std::vector<std::string> paths;
     emitSignal<DRing::ConfigurationSignal::GetAppDataPath>("config", &paths);
     if (not paths.empty())
         config_path = paths[0];
     return config_path;
 
-#elif defined( __APPLE__ )
-    std::string configdir = fileutils::get_home_dir() + DIR_SEPARATOR_STR
-        + "Library" + DIR_SEPARATOR_STR + "Application Support"
-        + DIR_SEPARATOR_STR + PACKAGE;
 #elif defined(RING_UWP)
     std::vector<std::string> paths;
     emitSignal<DRing::ConfigurationSignal::GetAppDataPath>("", &paths);
@@ -548,10 +572,15 @@ get_config_dir()
             RING_DBG("Cannot create directory: %s!", config_path.c_str());
     }
     return config_path;
+#else
+#if defined(__APPLE__)
+    std::string configdir = fileutils::get_home_dir() + DIR_SEPARATOR_STR
+        + "Library" + DIR_SEPARATOR_STR + "Application Support"
+        + DIR_SEPARATOR_STR + PACKAGE;
 #else
     std::string configdir = fileutils::get_home_dir() + DIR_SEPARATOR_STR +
                             ".config" + DIR_SEPARATOR_STR + PACKAGE;
-
+#endif
     const std::string xdg_env(XDG_CONFIG_HOME);
     if (not xdg_env.empty())
         configdir = xdg_env + DIR_SEPARATOR_STR + PACKAGE;
